﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;

namespace ComplexConverter
{
    public class EventArg : EventArgs
    {
        // Das Event ComplexConverter.Convert(), welches dieses EventArg verwendet, wird in zwei
        // Modi ausgeführt:
        // Store konvertiert einen gegebenen Wert und fügt der IList(Of String) einen Eintrag hinzu
        // Restore re-konvertiert einen Eintrag und überschreibt damit den gegebenen Wert
        private IList<string> _Strings;
        private int _Counter;
        public readonly bool IsStoring;

        public EventArg(IList<string> Strings, bool IsStoring)
        {
            this._Counter = 0;
            this._Strings = Strings;
            this.IsStoring = IsStoring;
        }

        /// <summary>
        /// Hier angegebene Werte werden gespeichert und restauriert
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="item"></param>
        [Description("Hier angegebene Werte werden gespeichert und restauriert")]
        public void ConvertValue<T>(ref T item)
        {
            if (this.IsStoring)
            {
                string s = TypeDescriptor.GetConverter(typeof(T)).ConvertToString(item);
                if (s.Contains("|"))
                {
                    string Msg = "Der zu speichernde String '" + s + "' darf nicht den ComplexConverter.Separator-Char (" + System.Convert.ToString(124, 16) + ") enthalten.";
                    throw new ArgumentException(Msg, "Item");
                }

                this._Strings.Add(s);
            }
            else
            {
                item = this.GetValue<T>();
            }
        }

        /// <summary>
        /// gibt einen gespeicherten Wert zurück
        /// </summary>
        [Description("gibt einen gespeicherten Wert zurück")]
        public T GetValue<T>()
        {
            if (this.IsStoring)
            {
                throw new Exception(".GetValue() darf nicht aufgerufen werden, wenn .IsStoring = True.");
            }

            T GetValue;
            try
            {
                TypeConverter C = TypeDescriptor.GetConverter(typeof(T));
                GetValue = (T)C.ConvertFromString(this._Strings[this._Counter]);
            }
            catch (Exception ex)
            {
                throw new RestoreException(ex);
            }

            this._Counter++;
            return GetValue;
        }

        // Sonderfall ConvertList(): Bei Listen wird die Anzahl Count der Elemente gespeichert.
        // Restore leert die Liste und fügt dann Count uninitialisierte Elemente hinzu.
        /// <summary>
        /// speichert Anzahl, restauriert mit uninitialisierten Elementen
        /// </summary>
        /// <typeparam name="TItem"></typeparam>
        /// <param name="list"></param>
        [Description("speichert Anzahl, restauriert mit uninitialisierten Listenelementen")]
        public void ConvertList<TItem>(IList list) where TItem : new()
        {
            if (this.IsStoring)
            {
                this._Strings.Add(list.Count.ToString());
                return;
            }

            list.Clear();
            try
            {
                for (int i = 0; i < int.Parse(this._Strings[this._Counter]); i++)
                {
                    list.Add(new TItem());
                }
            }
            catch (Exception ex)
            {
                throw new RestoreException(ex);
            }
            this._Counter++;
        }

        public void ConvertConverter(ComplexConverter Converter)
        {
            // ComplexConverters listigste Zeile: ermöglicht, den Konvertierungs-Vorgang an einen
            // anderen Converter abzugeben, dabei aber "meine" _Strings zu verwenden. So kann ein
            // UserControl aus einer anderen Assembly seine eigene Konvertierung definieren, die
            // dann in den Settings der Entry-Assembly mit-persistiert wird
            Converter.OnConvert(this);
        }
    }
}
